# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

class RenderingMethod(object):
    RASTER = 0
    ARROWS = 1
    BARBS = 2
    TRAJECTORIES = 3
    DOTSCLOUD = 4
    STREAMLINES = 5
    RAWRGB = 6

