# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

class CommandArgs(object):
    NONE = 0
    AddSource = 1
    RemoveSource = 2
    AddCollection = 3
    RemoveCollection = 4
    AddVariable = 5
    RemoveVariable = 6
    SetCurrentDatetime = 7
    AddTimespan = 8
    RemoveTimespan = 9
    SetTimespan = 10
    AddTimestep = 11
    RemoveTimestep = 12
    SetTimestep = 13
    SetNearestMode = 14
    SetDataAlwaysVisibleMode = 15
    SearchGranules = 16
    SetRendering = 17
    GetRendering = 18
    AddGranule = 19
    LookAt = 20
    SetAltitude = 21
    SelectVariable = 22
    SelectVariableByLabel = 23
    GetVariableIdentifier = 24
    CheckSchemaVersion = 25

