# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

class DataModel(object):
    LAT_LON = 0
    Y_X = 1
    NJ_NI = 2
    ROW_CELL = 3
    TIME_STATION = 4
    TIME = 5

