# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class RemoveCollection(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsRemoveCollection(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = RemoveCollection()
        x.Init(buf, n + offset)
        return x

    # RemoveCollection
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # RemoveCollection
    def CollectionId(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = o + self._tab.Pos
            from SEAScope.API.RenderableId import RenderableId
            obj = RenderableId()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

def RemoveCollectionStart(builder): builder.StartObject(1)
def RemoveCollectionAddCollectionId(builder, collectionId): builder.PrependStructSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(collectionId), 0)
def RemoveCollectionEnd(builder): return builder.EndObject()
