# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class GetExtractionResponse(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsGetExtractionResponse(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = GetExtractionResponse()
        x.Init(buf, n + offset)
        return x

    # GetExtractionResponse
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # GetExtractionResponse
    def Extractions(self, j):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            x = self._tab.Vector(o)
            x += flatbuffers.number_types.UOffsetTFlags.py_type(j) * 4
            x = self._tab.Indirect(x)
            from SEAScope.API.Extraction import Extraction
            obj = Extraction()
            obj.Init(self._tab.Bytes, x)
            return obj
        return None

    # GetExtractionResponse
    def ExtractionsLength(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.VectorLen(o)
        return 0

    # GetExtractionResponse
    def ExtractionsIsNone(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        return o == 0

def GetExtractionResponseStart(builder): builder.StartObject(1)
def GetExtractionResponseAddExtractions(builder, extractions): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(extractions), 0)
def GetExtractionResponseStartExtractionsVector(builder, numElems): return builder.StartVector(4, numElems, 4)
def GetExtractionResponseEnd(builder): return builder.EndObject()
