# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

class OpCode(object):
    addSource = 0
    removeSource = 1
    listSources = 2
    disconnect = 3
    addCollection = 4
    removeCollection = 5
    listCollections = 6
    addVariable = 7
    removeVariable = 8
    listVariables = 9
    setCurrentDatetime = 10
    getCurrentDatetime = 11
    addTimespan = 12
    removeTimespan = 13
    listTimespans = 14
    setTimespan = 15
    getTimespan = 16
    addTimestep = 17
    removeTimestep = 18
    listTimesteps = 19
    setTimestep = 20
    getTimestep = 21
    getNearestMode = 22
    setNearestMode = 23
    getDataAlwaysVisibleMode = 24
    setDataAlwaysVisibleMode = 25
    searchGranules = 26
    listVisibleData = 27
    setRendering = 28
    getRendering = 29
    getExtraction = 30
    addGranule = 31
    getCatalogues = 32
    resetCamera = 33
    zoomIn = 34
    zoomOut = 35
    setAltitude = 36
    lookAt = 37
    selectVariable = 38
    selectVariableByLabel = 39
    getVariableIdentifier = 40
    checkSchemaVersion = 41

