# automatically generated by the FlatBuffers compiler, do not modify

# namespace: API

import flatbuffers
from flatbuffers.compat import import_numpy
np = import_numpy()

class SetTimespan(object):
    __slots__ = ['_tab']

    @classmethod
    def GetRootAsSetTimespan(cls, buf, offset):
        n = flatbuffers.encode.Get(flatbuffers.packer.uoffset, buf, offset)
        x = SetTimespan()
        x.Init(buf, n + offset)
        return x

    # SetTimespan
    def Init(self, buf, pos):
        self._tab = flatbuffers.table.Table(buf, pos)

    # SetTimespan
    def Label(self):
        o = flatbuffers.number_types.UOffsetTFlags.py_type(self._tab.Offset(4))
        if o != 0:
            return self._tab.String(o + self._tab.Pos)
        return None

def SetTimespanStart(builder): builder.StartObject(1)
def SetTimespanAddLabel(builder, label): builder.PrependUOffsetTRelativeSlot(0, flatbuffers.number_types.UOffsetTFlags.py_type(label), 0)
def SetTimespanEnd(builder): return builder.EndObject()
